# encoding: utf-8
# Copyright (C) 2016 Ian Kelling

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
module B # blog module
  require 'fileutils'
  require 'time'
  require 'safe_yaml'
  require 'pygments'
  require 'sqlite3'
  require 'redcarpet'
  JS_INFO = "<p>All JavaScript has <a href=\"https://www.gnu.org/software/librejs/index.html\">LibreJS</a> support.</p>"

  DAY = 60*60*24
  DN = 'iankelling'
  FQDN = DN + '.org'
  DURL = 'https://' + FQDN
  DESCRIPTION = "Ian Kelling's personal site and blog on software"
  DATE_LEN = 'YYYY-MM-DD'.length
  NOW = Time.now.to_f
  WAIT_DATE = NOW - 60*60*24*1

  def db_init
    SQLite3::Database.new('../proposed-comments/comments.sqlite')
  end

  # from the redcarpet readme, then a bunch of googling to figure
  # out what to do on exception.
  class HTMLwithPygments < Redcarpet::Render::HTML
    def block_code(code, language)
      begin
        Pygments.highlight(code, lexer: language)
      rescue MentosError
        # when language detection fails
        Pygments.highlight(code, lexer: 'text')
      end
    end
  end

  def fwrite(output_path, string)
    output_path = File.join('./', output_path)
    FileUtils.mkdir_p(File.dirname(output_path))
    File.write(output_path, string)
  end

  def fskel(rel_path, title, content, o={})
    head = <<EOF
<link rel="canonical" href="#{DURL}/#{rel_path}">
EOF
    if rel_path =~ %r{^/blog/|^blog.html}
      head += <<EOF
<link rel="alternate" type="application/atom+xml" title="#{DN}" href="#{DURL}/feed.xml">
EOF
    end
    o[:head] = head
    fwrite(rel_path, skel(title, content, o))
  end
  def skel(title, content, o={})
    # got meta viewport from jekyll's default later. It's for better
    # mobile viewing.
    output = <<EOF
<!DOCTYPE html>
<html lang="en-US">
  <head>
    <meta charset="utf-8">
    <title>#{title}</title>
    <link rel="stylesheet" href="/css/main.css">
    <link rel="shortcut icon" href="/assets/favicon.png" />
    <meta name="description" content="#{o[:description] || DESCRIPTION}">
#{o[:head]}
    <meta name="viewport" content="width=device-width, initial-scale=1">
  </head>
  <body>
    <header class="page_header">
      <h3><a href="/">iankelling.org</a>#{o[:header]}</h3>
    </header>
    <div class="main-content-stripe">
      <div class="content">
        #{content}
      </div>
    </div>
    <div class="comment-stripe">
#{o[:comments]}
    </div>
    <footer>
#{o[:footer]}
      <p>This site has a <a href="/git/?p=iankelling.org;a=summary">git repo</a>. Unless stated otherwise, <a rel="license" href="http://creativecommons.org/licenses/by-sa/4.0/"><img class="cc-by-sa" alt="Creative Commons License" src="/assets/cc-by-sa-4.0-80x15.png" /></a></p>
      <p><address><a href="mailto:ian@iankelling.org">ian@iankelling.org</a> let me know what you think</address></p>
    </footer>
  </body>
</html>
EOF
    output
  end

  def stdpage(page_name, content)
    fskel("#{page_name}.html",
          "#{DN}/#{page_name}",
          content,
          header: " / <a href=\"/#{page_name}.html\">#{page_name}</a>")
  end

  def md_to_html(md)
    # Using redcarpet over kramdown because syntax highlighting is
    # simpler. kramdown uses some crap highlighter by default,
    # supports using rouge, but then the classes are all screwy
    # for what pygments css expects, rouge has a pygments compatibility mode,
    # but that is a pita to get working, then it doesn't even work right.
    # kramdown is jekyll's default markdown parser, but it doesn't use
    # it for code blocks, it strips them out using custom templating
    # extension class, then uses rouge, then wraps it in some
    # custom html for pygments compatibility. It's a complicated mess.
    Redcarpet::Markdown.new(HTMLwithPygments, fenced_code_blocks: true).render(md)
  end

  def comment_html(comment, date)
    # I tried putting the time,  %I:%M %p UTC, but it looks kinda
    # clunky, going against my simple theme.
    user_input = Redcarpet::Markdown.new(Redcarpet::Render::Safe,
                                   fenced_code_blocks: true).render(comment)
    <<EOF
<div class="comment">
  #{user_input}
  <p class="comment-date">#{Time.at(date).strftime("%b %-d '%y")}</p>
</div>
EOF
  end

  def post(file, build_time=false)
    content = File.read(file)
    content =~ %r{\A(---\s*\n.*?\n?)^((---)\s*$\n?)}m # yaml front matter
    # stuff after last match. jekyll uses $POSTMATCH,
    # but it's nil for me, I don't know what magic they are using.,
    # but only $' is listed here http://ruby-doc.org/core-2.3.1/doc/globals_rdoc.html,
    content = $'

    front = SafeYAML.load(Regexp.last_match(1))
    title = front['title']
    $page_title = "#{title} | #{DN}"
    header_rel = ' / <a href="/blog.html">blog</a> /'

    footer_extra = <<-EOF
<p><a class="icon-rss" href="/feed.xml">Subscribe</a></p>
EOF
    footer_extra += JS_INFO if content =~ /<script/



    b = File.basename(file,'.md')
    # date is in the format: YYYY-MM-DD-
    date = Time.parse(b[0..DATE_LEN])
    rel_path = "/blog/#{b[(DATE_LEN + 1)..-1]}.html"
    comments = $db.execute <<-SQL, [WAIT_DATE]
      select comment, date from c
      where page = '#{rel_path}' and (
       state = 'picked' or state = 'known' or state = 'timed'
       or (state = 'waiting' and date < ?))
SQL
    # get earliest comment. earlier ones stored in git will also be
    # published. This get's us easily sharable comments, and allows us
    # to expire unpublished comments and ip addresses which are PII and
    # should never be kept around indefinitely.
    sql_start_date = $db.execute('select min(date) from c')[0][0] || NOW
    comment_file_dir = "../comments/#{rel_path}"
    old_comments = Dir["#{comment_file_dir}/*"].reduce([]) do |memo, f|
      dt = File.basename(f).to_f
      if dt < sql_start_date
        memo << [File.read(f), dt]
      else
        FileUtils.rm(f) if build_time
        memo
      end
    end
    if build_time
      FileUtils.mkdir_p comment_file_dir
      comments.each do |c, c_date|
        # fyi: there is an extremely small chance of 2 comments having
        # the same floating point time and thus overwriting each other.
        # Small enough that it won't happen at my site's scale.
        File.write(File.join(comment_file_dir, c_date.to_s), c)
      end
      # https://piwik.org/docs/privacy/ says keep logs for 3-6 months
      $db.execute("delete from c where date < #{NOW - DAY*180}")
    end
    comments = old_comments + comments
    pending_comments = $db.execute(<<-SQL, [WAIT_DATE])[0][0]
      select count(*) from c
      where page = '#{rel_path}' and
       (state = 'waiting' and date > ? or state = 'suspect')
SQL

    feed_html = md_to_html(content)
    page_html = <<-EOF
      <header class="post-header">
        <h1 class="post-title">#{title}</h1>
        <p class="post-date">#{date.strftime("%b %-d, %Y")}</p>
      </header>
      #{feed_html}
EOF
    com_list = ''
    comments.each { |c, date| com_list += comment_html(c, date) }
    if pending_comments > 0
      if pending_comments >= 2
        text = "are #{pending_comments} new comments"
      else
        text = 'is 1 new comment'
      end
      com_list +=
        comment_html("Note: there #{text} pending approval.", NOW)
    end
    com_section = <<-EOF
      <form action="/cgi/comment" method="post">
        <input class="misc-comment-input" type="text" name="url">
        <input name="goto" type="hidden" value="#{rel_path}">
        <textarea rows="10" name="comment" placeholder="markdown" maxlength="1000"></textarea>
        <input type="submit" value="Leave a comment">
      </form>
      <div id="comments">
         #{com_list}
      </div>
EOF
    links = front['comment_links']
    if links
      link_html = links.map { |name,url| "<a href=\"#{url}\">#{name}</a>" }
                    .join(', ')
      com_section += (<<EOF)
<p>More comments at #{link_html}</p>
EOF
    end

    blog_toc_entry = "<li><a href=\"#{rel_path}\">#{title}</a></li>"

    com_section = <<EOF
<div id="comment-section" class="comment-section">
  #{com_section}
</div>
EOF


    if front['description']
      description = front['description']
    else
      # the first 300 saves ~ 1 ms
      # regex for striping html from liquid template src
      description = feed_html[0..300].gsub(/<script.*?<\/script>/m, '').
                      gsub(/<!--.*?-->/m, '').gsub(/<style.*?<\/style>/m, '').
                      gsub(/<.*?>/m, '')
      if description.length > 160
        description = description[0..156] + '...'
      end
    end

    head = <<EOF
EOF

    fskel(rel_path, title, page_html,
          header: header_rel,
          footer: footer_extra,
          comments: com_section,
          description: description)
    url="#{DURL}#{rel_path}"


    # following from https://creativecommons.org/choose,
    # with the addition of "unless otherwise noted", for js licenses.
    feed_copyright = <<-EOF
<a rel="license" href="http://creativecommons.org/licenses/by-sa/4.0/"><img alt="Creative Commons License" style="border-width:0" src="https://i.creativecommons.org/l/by-sa/4.0/88x31.png" /></a><br /><span xmlns:dct="http://purl.org/dc/terms/" href="http://purl.org/dc/dcmitype/Text" property="dct:title" rel="dct:type">#{title}</span> by <a xmlns:cc="http://creativecommons.org/ns#" href="#{url}" property="cc:attributionName" rel="cc:attributionURL">Ian Kelling</a> unless otherwise noted is licensed under a <a rel="license" href="http://creativecommons.org/licenses/by-sa/4.0/">Creative Commons Attribution-ShareAlike 4.0 International License</a>.
EOF

    feed_entry = <<EOF
<entry>
  <title>#{title}</title>
  <link rel="alternate" href="#{url}"/>
  <id>#{url}</id>
  <updated>#{date.to_datetime.rfc3339}</updated>
  <content type="html" xml:lang="en-us" xml:base="#{DURL}/blog">
    <![CDATA[
    #{feed_html}
    ]]>
  </content>
  <rights>
    #{feed_copyright}
  </rights>
</entry>
EOF
    return [feed_entry, blog_toc_entry]
  end
end
